#!/bin/sh
#------------------------------------------------------------------------------
# /opt/files/usr/local/bin/ssmpt.sh                                        v0.1 
#
# Creation:     2003-05-02  t_klingl
# Last Update:  2004-01-22  t_klingl
#------------------------------------------------------------------------------

#---------------------------------------
# Hilfsfunktionen
#---------------------------------------
#---------------------------------------
# usage ()
#---------------------------------------
usage ()
{
	echo "usage: ssmtp.sh -m MIME_Mailfile"
	echo "       ssmtp.sh -f from"
	echo "                -t to" 
	echo "               [-cc comma_separated_cc_address_list]"
	echo "               [-bcc comma_separated_bcc_address_list]"
	echo "                -r \"reply-to\""
	echo "                -s \"subject\""
	echo "                -b \"mailbody_text\" | \"/absolute/path/to/body.utf8\""
	echo "               [-d directory [-rp regex_positiv] [-rn regex_negativ]]"
	echo "               [[MIME-Type:]attachment] ..."
}

#---------------------------------------
# getArg (string, string)
#---------------------------------------
getArg ()
{
	ARG1=$1
	ARG2=$2
	
	if [ "${ARG2}" != "" ];
	then
		case "${ARG1}"
		in
			-m) FILE="${ARG2}";;
			-f) FROM="${ARG2}";;
			-r) REPLYTO="${ARG2}";;
			-t) TO="${ARG2}";;
			-s) SUBJECT="${ARG2}";;
			-b) BODY="${ARG2}";;
			-cc) CC="${ARG2}";;
			-bcc) BCC="${ARG2}";;
			-d) DIR="${ARG2}";;
			-rp) REGEXPOS="${ARG2}";;
			-rn) REGEXNEG="${ARG2}";;
			 *) break 2;;
		esac
		return 0
	fi
	return 1	
}

#---------------------------------------
# log (string)
#---------------------------------------
log ()
{
	TEXT=$1

	if [ "${SSMTP_LOGFILE}" != "" ]; then
		case "${SSMTP_LOGFILE}"
		in
			syslog) logger "ssmtp: ${TEXT}";;
			*)      echo "${DATE} ssmtp: ${TEXT}" >> ${SSMTP_LOGFILE};;
		esac
	fi
    
	return 1	
}

#---------------------------------------
# attach (string)
#---------------------------------------
attach ()
{
	FILES=$1
	
	HELP1=`echo "${FILES}" | sed 's/^\(.*\):\(.*\)/\1/g'`
	HELP2=`echo "${FILES}" | sed 's/^\(.*\):\(.*\)/\2/g'`

	#echo "Help1=${HELP1}"
	#echo "Help2=${HELP2}"
	if [ "${HELP1}" = "${HELP2}" ]; then
		# kein ContentType angegeben
		# falls vorhanden, 'file' zur Bestimmung nutzen
		if [ "${FILECMD}" = "1" ]; then
			CONTENTTYPE=`/usr/local/bin/file -i "${FILES}" | cut -d":" -f 2-`
		else
			CONTENTTYPE=${DEFAULTCT}
		fi
	else
		# angegebener ContentType hat Vorrang
		CONTENTTYPE="${HELP1}"
		FILES="${HELP2}"
	fi
	
	BASENAME=`basename "${FILES}"`
	{
		echo -e "${BOUNDARY}"
		echo -e "Content-Type:${CONTENTTYPE}; name=\"${BASENAME}\""
		echo -e "Content-Disposition: attachment"
		echo -e "Content-Transfer-Encoding: base64"
		echo -e ""
	} >> ${SPOOLFILE}
	
	mmencode "${FILES}" >> ${SPOOLFILE}
	
	{
		echo -e ""
	} >> ${SPOOLFILE}
	
	return 1
}

#---------------------------------------
# Parameter bestimmen und einlesen
#---------------------------------------
TO=''
FROM=''
REPLYTO=''
FILE=''
SUBJECT=''
BODY=''
CC=''
BCC=''
DIR=''
REGEXPOS=''
REGEXNEG=''

while test $# -gt 0
do
	getArg "$1" "$2"
	if [ "$?" = "1" ];
	then
		break;
	fi
	shift
	shift
done

if [ "${FILE}" != "" ];
then
	if [ "${FROM}" != "" -o "${TO}" != "" -o "${SUBJECT}" != "" -o "${BODY}" != "" ];
	then
		usage
		exit 1
	fi
else
	if [ "${FROM}" = "" -o "${TO}" = "" -o "${SUBJECT}" = "" -o "${BODY}" = "" ];
	then
		usage
		exit 1
	fi
fi

#---------------------------------------
# Variablen inkludieren
#---------------------------------------

. /etc/ssmtp/ssmtp_skripte.conf

#---------------------------------------
# Hilfsvariablen
#---------------------------------------

TIMESTAMP=`date +%Y%m%d%H%M%S`      # als eineindeutigen Dateinamen verwenden
DATE=`date '+%a, %d %b %Y %H:%M:%S %z'` 
                                    # fuer das Datumsfeld der EMail
SPOOLFILE="${SSMTP_SPOOL_DIR}/${TIMESTAMP}.mail"
                                    # eindeutiger Name des Spoolfiles
SPOOLFILE_SHORT="${SSMTP_SPOOL_DIR}/${TIMESTAMP}"
                                    # eindeutiger Kurzname des Spoolfiles
BOUNDARY="--========Boundary155661039885988"
                                    # Abschnittbegrenzer
DEFAULTCT="application/octet-stream"
                                    # falls kein MIME-Type angegeben und 'file' nicht vorhanden ist
DEFAULTREGEXPOS="^.*$"
                                    # regulaerer Ausdruck (positiv), matcht auf alle Dateinamen
DEFAULTREGEXNEG="^$"
                                    # regulaerer Ausdruck (negativ), matcht auf keinen Dateinamen

REGEXPOSFILE="${SSMTP_SPOOL_DIR}/${TIMESTAMP}.regexpos"
REGEXNEGFILE="${SSMTP_SPOOL_DIR}/${TIMESTAMP}.regexno"
                                    
#-----------------
# sind 'file', "egrep" und "uuidgen"-Hilfprogramme vorhanden ?
# wir gehen davon aus, wenn OPT_SSMTPPRO gesetzt ist

if [ "`echo ${SSMTP_PRO} | grep -i '^yes$'`" = "" ];
then
	FILECMD="0"
	GREPCMD="grep"
	EGREP="0"
	GUID=${TIMESTAMP}
else
	FILECMD="1"
	GREPCMD="egrep -E -f"
	EGREP="1"
	GUID=`uuidgen`
fi

#---------------------------------------
# Variablen einlesen & belegen
#---------------------------------------

if [ "${FILE}" = "" ]; then
	#---------------------------------------
	# zweite Aufrufvariante
	#---------------------------------------
	
	if [ "$1" != "" ]; then
		ATTACHMENTS=1
	else
		ATTACHMENTS=0
	fi
	
	if [ "${REGEXPOS}" = "" ]; then
		REGEXPOS="${DEFAULTREGEXPOS}"
	fi
	echo "${REGEXPOS}" | cat > ${REGEXPOSFILE}
	
	if [ "${REGEXNEG}" = "" ]; then
		REGEXNEG="${DEFAULTREGEXNEG}"
	fi
	echo "${REGEXNEG}" | cat > ${REGEXNEGFILE}
	
	#Mailfile erstellen
	{
		if [ "`echo "${FROM}" | grep '^<.*>$'`" != "" ]; then
			echo -e "From: ${FROM}"
		elif [ "`echo "${FROM}" | grep '<.*>$'`" != "" ]; then
			MAIL=`echo "${FROM}" | sed 's/\(.*\)\(<.*>\)/\2/g'`
			NAME=`echo "${FROM}" | sed 's/\(.*\)\( <.*>\)/\1/g' | sed 's/\"//g' | mmencode -q | sed 's/ /_/g'`
			echo -e "From: =?utf-8?Q?${NAME}?= ${MAIL}"
		else
			echo -e "From: <${FROM}>"
		fi
		if [ "`echo "${TO}" | grep '^<.*>$'`" != "" ]; then
			echo -e "To: ${TO}"
		elif [ "`echo "${TO}" | grep '<.*>$'`" != "" ]; then
			MAIL=`echo "${TO}" | sed 's/\(.*\)\(<.*>\)/\2/g'`
			NAME=`echo "${TO}" | sed 's/\(.*\)\( <.*>\)/\1/g' | sed 's/\"//g' | mmencode -q | sed 's/ /_/g'`
			echo -e "To: =?utf-8?Q?${NAME}?= ${MAIL}"
		else
			echo -e "To: <${TO}>"
		fi
		echo -e "Date: ${DATE}"
		echo -e "Subject: =?UTF-8?Q?`echo ${SUBJECT} | mmencode -q`?="
		echo -e "X-Sender: opt_ssmtp"
		echo -e "Message-ID: <${GUID}@${SSMTP_HOSTNAME}>"
	} > ${SPOOLFILE}
	
	if [ "${REPLYTO}" != "" ]; then
	{
                if [ "`echo "${REPLYTO}" | grep '^<.*>$'`" != "" ]; then
                        echo -e "Reply-To: ${REPLYTO}"
                elif [ "`echo "${REPLYTO}" | grep '<.*>$'`" != "" ]; then
                        MAIL=`echo "${REPLYTO}" | sed 's/\(.*\)\(<.*>\)/\2/g'`
                        NAME=`echo "${REPLYTO}" | sed 's/\(.*\)\( <.*>\)/\1/g' | sed 's/\"//g' | mmencode -q | sed 's/ /_/g'`
                        echo -e "Reply-To: =?utf-8?Q?${NAME}?= ${MAIL}"
                else
                        echo -e "Reply-To: <${REPLYTO}>"
                fi
	} >> ${SPOOLFILE}
	fi
	
	if [ "${CC}" != "" ];
	then
		CC="<${CC}>"
		echo -e "Cc: `echo \"${CC}\" | sed 's/,/>,\
 </g'`" >> ${SPOOLFILE}
	fi
	
	if [ "${BCC}" != "" ];
	then
		BCC="<${BCC}>"
		echo -e "Cc: `echo \"${BCC}\" | sed 's/,/>,\
 </g'`" >> ${SPOOLFILE}
	fi
	
	{
		echo -e "MIME-Version: 1.0"
		echo -e "Content-Type: multipart/mixed; boundary=\"`echo ${BOUNDARY} | sed 's/^..\(.*\)$/\1/g'`\""
		echo -e ""
		echo -e "This is a MIME encapsulated multipart message -"
		echo -e "please use a MIME-compliant e-mail program to open it."
		echo -e ""
		echo -e "${BOUNDARY}"
		echo -e "Content-Type: text/plain; charset=\"UTF-8\""
		echo -e "Content-Transfer-Encoding: quoted-printable"
		echo -e ""
	} >> ${SPOOLFILE}
	
	if [ "`echo ${BODY} | cut -c 1`" = "/" ];
	then
		# Datei inkludieren
		cat "${BODY}" | mmencode -q >> ${SPOOLFILE}
	else
		echo "${BODY}" | mmencode -q >> ${SPOOLFILE}
	fi
	
	{
		echo -e ""
	} >> ${SPOOLFILE}
	
	if [ "$ATTACHMENTS" = "1" ]; then
		
		for FILES
		do
			attach "${FILES}"
		done
	
	fi
	
	if [ "${DIR}" != "" ]; then

	    for FILES in `ls "${DIR}"`
	    do
	        if [ "${EGREP}" = "1" ]; then
	        	FILES=`echo "${FILES}" | ${GREPCMD} ${REGEXPOSFILE}`
	        else
	         	FILES=`echo "${FILES}" | ${GREPCMD} ${REGEXPOS}`
	        fi
	        if [ "${FILES}" != "" ]; then
	            if [ "${EGREP}" = "1" ]; then
	            	RESULT=`echo "${FILES}" | ${GREPCMD} ${REGEXNEGFILE}`
	            else
	            	RESULT=`echo "${FILES}" | ${GREPCMD} ${REGEXNEG}`
	            fi
	            if [ "${RESULT}" != "" ]; then
	                FILES=""
	            fi
	        fi
	        if [ "${FILES}" != "" ]; then
	            #echo "wird angehaengt : ${FILES}"
	            attach "${FILES}"
	        fi
	    done
	    
	fi
	
	{
		echo -e "${BOUNDARY}--"
		echo -e ""
	} >> ${SPOOLFILE}

	rm ${REGEXPOSFILE}
	rm ${REGEXNEGFILE}
	chmod 666 ${SPOOLFILE}
	#---------------------------------------
	# Ende zweite Aufrufvariante
	#---------------------------------------

else

	#---------------------------------------
	# erste Aufrufvariante
	#---------------------------------------
	if [ "${FILE}" = "" ]; then
		usage
		exit 1;
	fi
	# ins SPOOL-Verzeichnis kopieren
	cp ${FILE} ${SPOOLFILE}
	chmod 666 ${SPOOLFILE}

	#---------------------------------------
	# Ende erste Aufrufvariante
	#---------------------------------------
fi	

#---------------------------------------
# Sofort versenden ?
#---------------------------------------

if [ "${SSMTP_SEND_AT_ONCE}" = "yes" ];
then
	/usr/local/bin/ssmtp_send.sh "${SPOOLFILE}"
	if [ "$?" = "0" ]; then
		# alles OK
		log "Mail ${SPOOLFILE} sent successfully"
		rm "${SPOOLFILE}"
		exit 0
	else
		log "Error sending mail ${SPOOLFILE}, error $?, first attempt"
		mv ${SPOOLFILE} ${SPOOLFILE_SHORT}.retry1
	fi
else
	# erst spaeter versenden
	mv "${SPOOLFILE}" "${SPOOLFILE}.retry0"
fi

exit 0

